@@ -1,7 +1,7 @@
 /*
  * filters.c	Routines to parse Ascend's filter attributes.
  *
- * Version:	$Id: filters.c,v 1.52 2007/11/25 14:02:07 aland Exp $
+ * Version:	$Id: 4868cd385d9faaff1495cf79d2bfd75866884a91 $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,11 +20,11 @@
  * Copyright 2003,2006  The FreeRADIUS server project
  */
 
-#include <freeradius-devel/ident.h>
-RCSID("$Id: filters.c,v 1.52 2007/11/25 14:02:07 aland Exp $")
+RCSID("$Id: 4868cd385d9faaff1495cf79d2bfd75866884a91 $")
 
 #include <freeradius-devel/libradius.h>
 
+#ifdef WITH_ASCEND_BINARY
 #include <ctype.h>
 
 /*
@@ -48,9 +48,9 @@ RCSID("$Id: filters.c,v 1.52 2007/11/25 14:02:07 aland Exp $")
 
 #define IPX_NODE_ADDR_LEN		6
 
-#if ! defined( FALSE )
-# define FALSE		0
-# define TRUE		(! FALSE)
+#if ! defined( false )
+# define false		0
+# define true		(! false)
 #endif
 
 
@@ -72,8 +72,8 @@ RCSID("$Id: filters.c,v 1.52 2007/11/25 14:02:07 aland Exp $")
  *
  *	proto:		The IP protocol number
  *
- *	established:	A boolean value.  TRUE when we care about the
- *			established state of a TCP connection.  FALSE when
+ *	established:	A boolean value.  true when we care about the
+ *			established state of a TCP connection.  false when
  *			we dont care.
  *
  *	srcport:	TCP or UDP source port number.
@@ -101,7 +101,7 @@ typedef struct ascend_ip_filter_t {
 	uint16_t	dstport;
 	uint8_t		srcPortComp;
 	uint8_t		dstPortComp;
-	unsigned char   fill[4];        /* used to be fill[2] */
+	unsigned char   fill[4];	/* used to be fill[2] */
 } ascend_ip_filter_t;
 
 
@@ -173,7 +173,7 @@ typedef struct ascend_generic_filter_t {
 	uint8_t		mask[ RAD_MAX_FILTER_LEN ];
 	uint8_t		value[ RAD_MAX_FILTER_LEN ];
 	uint8_t		compNeq;
-	uint8_t		fill[3];        /* used to be fill[1] */
+	uint8_t		fill[3];	/* used to be fill[1] */
 } ascend_generic_filter_t;
 
 /*
@@ -186,11 +186,11 @@ typedef struct ascend_generic_filter_t {
  *
  *	type:		Either RAD_FILTER_GENERIC or RAD_FILTER_IP.
  *
- *	forward:	TRUE if we should forward packets that match this
- *			filter, FALSE if we should drop packets that match
+ *	forward:	true if we should forward packets that match this
+ *			filter, false if we should drop packets that match
  *			this filter.
  *
- *	direction:	TRUE if this is an input filter, FALSE if this is
+ *	direction:	true if this is an input filter, false if this is
  *			an output filter.
  *
  *	fill:		Round things out to a dword boundary.
@@ -349,31 +349,6 @@ static const FR_NAME_NUMBER filterCompare[] = {
 };
 
 
-/*
- *	String split routine.  Splits an input string IN PLACE
- *	into pieces, based on spaces.
- */
-static int str2argv(char *str, char **argv, int max_argc)
-{
-	int argc = 0;
-
-	while (*str) {
-		if (argc >= max_argc) return argc;
-
-		while (*str == ' ') *(str++) = '\0';
-
-		if (!*str) return argc;
-
-		argv[argc] = str;
-		argc++;
-
-		while (*str && (*str != ' ')) str++;
-	}
-
-	return argc;
-}
-
-
 /*
  *	ascend_parse_ipx_net
  *
@@ -383,7 +358,7 @@ static int ascend_parse_ipx_net(int argc, char **argv,
 				ascend_ipx_net_t *net, uint8_t *comp)
 {
 	int		token;
-	const char	*p;
+	char const	*p;
 
 	if (argc < 3) return -1;
 
@@ -415,7 +390,7 @@ static int ascend_parse_ipx_net(int argc, char **argv,
 	/*
 	 *	Node must be 6 octets long.
 	 */
-	token = fr_hex2bin(p, net->node, IPX_NODE_ADDR_LEN);
+	token = fr_hex2bin(net->node, IPX_NODE_ADDR_LEN, p, strlen(p));
 	if (token != IPX_NODE_ADDR_LEN) return -1;
 
 	/*
@@ -486,17 +461,17 @@ static int ascend_parse_ipx_net(int argc, char **argv,
  *	where:
  *
  *  srcipxnet:      Keyword for source IPX address.
- *                  nnnn = IPX Node address.
+ *		  nnnn = IPX Node address.
  *
  *  srcipxnode:     Keyword for source IPX Node address.
- *                  mmmmm = IPX Node Address, could be FFFFFF.
- *                  A vlid ipx node number should accompany ipx net number.
+ *		  mmmmm = IPX Node Address, could be FFFFFF.
+ *		  A vlid ipx node number should accompany ipx net number.
  *
  *	srcipxsoc:      Keyword for source IPX socket address.
  *
- *	cmd:            One of ">" or "<" or "=" or "!=".
+ *	cmd:	    One of ">" or "<" or "=" or "!=".
  *
- *	value:          Socket value to be compared against, in hex.
+ *	value:	  Socket value to be compared against, in hex.
  *
  *	dstipxnet:	Keyword for destination IPX address.
  *			nnnn = IPX Node address.
@@ -553,7 +528,7 @@ static int ascend_parse_ipx(int argc, char **argv, ascend_ipx_filter_t *filter)
 			break;
 
 		default:
-			librad_log("Unknown string \"%s\" in IPX data filter",
+			fr_strerror_printf("Unknown string \"%s\" in IPX data filter",
 				   argv[0]);
 			return -1;
 		}
@@ -583,7 +558,7 @@ static int ascend_parse_ipaddr(uint32_t *ipaddr, char *str)
 {
 	int		count = 0;
 	int		ip[4];
-	int             masklen;
+	int	     masklen;
 	uint32_t	netmask = 0;
 
 	/*
@@ -623,10 +598,9 @@ static int ascend_parse_ipaddr(uint32_t *ipaddr, char *str)
 				str += strspn(str, "0123456789");
 				netmask = masklen;
 				goto finalize;
-				break;
 
 			default:
-				librad_log("Invalid character in IP address");
+				fr_strerror_printf("Invalid character in IP address");
 				return -1;
 			}
 		} /* loop over one character */
@@ -707,7 +681,7 @@ static int ascend_parse_port(uint16_t *port, char *compare, char *str)
 #define IP_SRC_PORT_FLAG    (1 << 2)
 #define IP_DEST_PORT_FLAG   (1 << 3)
 #define IP_PROTO_FLAG       (1 << 4)
-#define IP_EST_FLAG         (1 << 5)
+#define IP_EST_FLAG	 (1 << 5)
 
 #define DONE_FLAGS	(IP_SRC_ADDR_FLAG | IP_DEST_ADDR_FLAG | \
 			IP_SRC_PORT_FLAG | IP_DEST_PORT_FLAG | \
@@ -831,7 +805,7 @@ static int ascend_parse_ip(int argc, char **argv, ascend_ip_filter_t *filter)
 			} else {
 				token = fr_str2int(filterProtoName, argv[0], -1);
 				if (token == -1) {
-					librad_log("Unknown IP protocol \"%s\" in IP data filter",
+					fr_strerror_printf("Unknown IP protocol \"%s\" in IP data filter",
 						   argv[0]);
 					return -1;
 				}
@@ -849,7 +823,7 @@ static int ascend_parse_ip(int argc, char **argv, ascend_ip_filter_t *filter)
 	 *	We should have parsed everything by now.
 	 */
 	if (argc != 0) {
-		librad_log("Unknown extra string \"%s\" in IP data filter",
+		fr_strerror_printf("Unknown extra string \"%s\" in IP data filter",
 			   argv[0]);
 		return -1;
 	}
@@ -914,17 +888,12 @@ static int ascend_parse_generic(int argc, char **argv,
 	filter->offset = rcode;
 	filter->offset = htons(filter->offset);
 
-	rcode = fr_hex2bin(argv[1], filter->mask, sizeof(filter->mask));
+	rcode = fr_hex2bin(filter->mask, sizeof(filter->mask), argv[1], strlen(argv[1]));
 	if (rcode != sizeof(filter->mask)) return -1;
 
-	token = fr_hex2bin(argv[2], filter->value, sizeof(filter->value));
+	token = fr_hex2bin(filter->value, sizeof(filter->value), argv[2], strlen(argv[2]));
 	if (token != sizeof(filter->value)) return -1;
 
-	/*
-	 *	The mask and value MUST be the same length.
-	 */
-	if (rcode != token) return -1;
-
 	filter->len = rcode;
 	filter->len = htons(filter->len);
 
@@ -942,12 +911,12 @@ static int ascend_parse_generic(int argc, char **argv,
 		switch (token) {
 		case FILTER_GENERIC_COMPNEQ:
 			if (flags & 0x01) return -1;
-			filter->compNeq = TRUE;
+			filter->compNeq = true;
 			flags |= 0x01;
 			break;
 		case FILTER_GENERIC_COMPEQ:
 			if (flags & 0x01) return -1;
-			filter->compNeq = FALSE;
+			filter->compNeq = false;
 			flags |= 0x01;
 			break;
 
@@ -958,7 +927,7 @@ static int ascend_parse_generic(int argc, char **argv,
 			break;
 
 		default:
-			librad_log("Invalid string \"%s\" in generic data filter",
+			fr_strerror_printf("Invalid string \"%s\" in generic data filter",
 				   argv[0]);
 			return -1;
 		}
@@ -971,44 +940,47 @@ static int ascend_parse_generic(int argc, char **argv,
 }
 
 
-/*
- * filterBinary:
+/** Filter binary
  *
  * This routine will call routines to parse entries from an ASCII format
  * to a binary format recognized by the Ascend boxes.
  *
- *	pair:			Pointer to value_pair to place return.
- *
- *	valstr:			The string to parse
- *
- *	return:			-1 for error or 0.
+ * @param out Where to write parsed filter.
+ * @param value ascend filter text.
+ * @param len of value.
+ * @return -1 for error or 0.
  */
-int
-ascend_parse_filter(VALUE_PAIR *pair)
+int ascend_parse_filter(value_data_t *out, char const *value, size_t len)
 {
 	int		token, type;
-	int	        rcode;
+	int		rcode;
 	int		argc;
 	char		*argv[32];
 	ascend_filter_t filter;
+	char		*p;
 
 	rcode = -1;
 
+	/*
+	 *	Tokenize the input string in the VP.
+	 *
+	 *	Once the filter is *completely* parsed, then we will
+	 *	over-write it with the final binary filter.
+	 */
+	p = talloc_bstrndup(NULL, value, len);
+
 	/*
 	 *	Rather than printing specific error messages, we create
 	 *	a general one here, which won't be used if the function
 	 *	returns OK.
 	 */
-	librad_log("Text is not in proper format");
+	fr_strerror_printf("Failed parsing \"%s\" as ascend filer", p);
 
-	/*
-	 *	Tokenize the input string in the VP.
-	 *
-	 *	Once the filter is *completelty* parsed, then we will
-	 *	over-write it with the final binary filter.
-	 */
-	argc = str2argv(pair->vp_strvalue, argv, 32);
-	if (argc < 3) return -1;
+	argc = str2argv(p, argv, 32);
+	if (argc < 3) {
+		talloc_free(p);
+		return -1;
+	}
 
 	/*
 	 *	Decide which filter type it is: ip, ipx, or generic
@@ -1027,9 +999,9 @@ ascend_parse_filter(VALUE_PAIR *pair)
 		break;
 
 	default:
-		librad_log("Unknown Ascend filter type \"%s\"", argv[0]);
+		fr_strerror_printf("Unknown Ascend filter type \"%s\"", argv[0]);
+		talloc_free(p);
 		return -1;
-		break;
 	}
 
 	/*
@@ -1046,9 +1018,9 @@ ascend_parse_filter(VALUE_PAIR *pair)
 		break;
 
 	default:
-		librad_log("Unknown Ascend filter direction \"%s\"", argv[1]);
+		fr_strerror_printf("Unknown Ascend filter direction \"%s\"", argv[1]);
+		talloc_free(p);
 		return -1;
-		break;
 	}
 
 	/*
@@ -1065,16 +1037,15 @@ ascend_parse_filter(VALUE_PAIR *pair)
 		break;
 
 	default:
-		librad_log("Unknown Ascend filter action \"%s\"", argv[2]);
+		fr_strerror_printf("Unknown Ascend filter action \"%s\"", argv[2]);
+		talloc_free(p);
 		return -1;
-		break;
 	}
 
 
 	switch (type) {
 	case RAD_FILTER_GENERIC:
-		rcode = ascend_parse_generic(argc - 3, &argv[3],
-					  &filter.u.generic);
+		rcode = ascend_parse_generic(argc - 3, &argv[3], &filter.u.generic);
 		break;
 
 	case RAD_FILTER_IP:
@@ -1084,53 +1055,15 @@ ascend_parse_filter(VALUE_PAIR *pair)
 	case RAD_FILTER_IPX:
 		rcode = ascend_parse_ipx(argc - 3, &argv[3], &filter.u.ipx);
 		break;
-
-	default:		/* should never reach here. */
-		break;
 	}
 
 	/*
 	 *	Touch the VP only if everything was OK.
 	 */
-	if (rcode == 0) {
-		pair->length = sizeof(filter);
-		memcpy(pair->vp_filter, &filter, sizeof(filter));
-	}
+	if (rcode == 0) memcpy(out->filter, &filter, sizeof(filter));
+	talloc_free(p);
 
 	return rcode;
-
-#if 0
-    /*
-     * if 'more' is set then this new entry must exist, be a
-     * FILTER_GENERIC_TYPE, direction and disposition must match for
-     * the previous 'more' to be valid. If any should fail then TURN OFF
-     * previous 'more'
-     */
-    if( prevRadPair ) {
-	filt = ( RadFilter * )prevRadPair->vp_strvalue;
-	if(( tok != FILTER_GENERIC_TYPE ) || (rc == -1 ) ||
-	   ( prevRadPair->attribute != pair->attribute ) ||
-	   ( filt->indirection != radFil.indirection ) ||
-	   ( filt->forward != radFil.forward ) ) {
-	    gen = &filt->u.generic;
-	    gen->more = FALSE;
-	    librad_log("filterBinary:  'more' for previous entry doesn't match: %s.\n",
-		     valstr);
-	}
-    }
-    prevRadPair = NULL;
-    if( rc != -1 && tok == FILTER_GENERIC_TYPE ) {
-	if( radFil.u.generic.more ) {
-	    prevRadPair = pair;
-	}
-    }
-
-    if( rc != -1 ) {
-	memcpy( pair->vp_strvalue, &radFil, pair->length );
-    }
-    return(rc);
-
-#endif
 }
 
 /*
@@ -1141,177 +1074,174 @@ ascend_parse_filter(VALUE_PAIR *pair)
  *	Note we don't bother checking 'len' after the snprintf's.
  *	This function should ONLY be called with a large (~1k) buffer.
  */
-void print_abinary(VALUE_PAIR *vp, char *buffer, size_t len)
+void print_abinary(char *out, size_t outlen, uint8_t const *data, size_t len, int8_t quote)
 {
-  size_t 		i;
-  char			*p;
-  ascend_filter_t	*filter;
+	size_t 	i;
+	char	*p;
+	ascend_filter_t	const *filter;
 
-  static const char *action[] = {"drop", "forward"};
-  static const char *direction[] = {"out", "in"};
+	static char const *action[] = {"drop", "forward"};
+	static char const *direction[] = {"out", "in"};
 
-  p = buffer;
+	p = out;
 
-  /*
-   *  Just for paranoia: wrong size filters get printed as octets
-   */
-  if (vp->length != sizeof(*filter)) {
-	  strcpy(p, "0x");
-	  p += 2;
-	  len -= 2;
-	  for (i = 0; i < vp->length; i++) {
-		  snprintf(p, len, "%02x", vp->vp_octets[i]);
-		  p += 2;
-		  len -= 2;
-	  }
-	  return;
-  }
+	/*
+	 *  Just for paranoia: wrong size filters get printed as octets
+	 */
+	if (len != sizeof(*filter)) {
+		strcpy(p, "0x");
+		p += 2;
+		outlen -= 2;
+		for (i = 0; i < len; i++) {
+			snprintf(p, outlen, "%02x", data[i]);
+			p += 2;
+			outlen -= 2;
+		}
+		return;
+	}
 
-  *(p++) = '"';
-  len -= 3;			/* account for leading & trailing quotes */
+	if (quote > 0) {
+		*(p++) = (char) quote;
+		outlen -= 3;			/* account for leading & trailing quotes */
+	}
 
-  filter = (ascend_filter_t *) &(vp->vp_filter);
-  i = snprintf(p, len, "%s %s %s",
-	       fr_int2str(filterType, filter->type, "??"),
-	       direction[filter->direction & 0x01],
-	       action[filter->forward & 0x01]);
+	filter = (ascend_filter_t const *) data;
+	i = snprintf(p, outlen, "%s %s %s", fr_int2str(filterType, filter->type, "??"),
+		     direction[filter->direction & 0x01], action[filter->forward & 0x01]);
 
-  p += i;
-  len -= i;
-
-  /*
-   *	Handle IP filters
-   */
-  if (filter->type == RAD_FILTER_IP) {
-
-    if (filter->u.ip.srcip) {
-      i = snprintf(p, len, " srcip %d.%d.%d.%d/%d",
-		   ((uint8_t *) &filter->u.ip.srcip)[0],
-		   ((uint8_t *) &filter->u.ip.srcip)[1],
-		   ((uint8_t *) &filter->u.ip.srcip)[2],
-		   ((uint8_t *) &filter->u.ip.srcip)[3],
-		   filter->u.ip.srcmask);
-      p += i;
-      len -= i;
-    }
-
-    if (filter->u.ip.dstip) {
-      i = snprintf(p, len, " dstip %d.%d.%d.%d/%d",
-		   ((uint8_t *) &filter->u.ip.dstip)[0],
-		   ((uint8_t *) &filter->u.ip.dstip)[1],
-		   ((uint8_t *) &filter->u.ip.dstip)[2],
-		   ((uint8_t *) &filter->u.ip.dstip)[3],
-		   filter->u.ip.dstmask);
-      p += i;
-      len -= i;
-    }
-
-    i =  snprintf(p, len, " %s",
-		  fr_int2str(filterProtoName, filter->u.ip.proto, "??"));
-    p += i;
-    len -= i;
-
-    if (filter->u.ip.srcPortComp > RAD_NO_COMPARE) {
-      i = snprintf(p, len, " srcport %s %d",
-		   fr_int2str(filterCompare, filter->u.ip.srcPortComp, "??"),
-		   ntohs(filter->u.ip.srcport));
-      p += i;
-      len -= i;
-    }
-
-    if (filter->u.ip.dstPortComp > RAD_NO_COMPARE) {
-      i = snprintf(p, len, " dstport %s %d",
-		   fr_int2str(filterCompare, filter->u.ip.dstPortComp, "??"),
-		   ntohs(filter->u.ip.dstport));
-      p += i;
-      len -= i;
-    }
-
-    if (filter->u.ip.established) {
-      i = snprintf(p, len, " est");
-      p += i;
-      len -= i;
-    }
-
-    /*
-     *	Handle IPX filters
-     */
-  } else if (filter->type == RAD_FILTER_IPX) {
-    /* print for source */
-    if (filter->u.ipx.src.net) {
-      i = snprintf(p, len, " srcipxnet 0x%04x srcipxnode 0x%02x%02x%02x%02x%02x%02x",
-		  (unsigned int)ntohl(filter->u.ipx.src.net),
-		  filter->u.ipx.src.node[0], filter->u.ipx.src.node[1],
-		  filter->u.ipx.src.node[2], filter->u.ipx.src.node[3],
-		  filter->u.ipx.src.node[4], filter->u.ipx.src.node[5]);
-      p += i;
-      len -= i;
-
-      if (filter->u.ipx.srcSocComp > RAD_NO_COMPARE) {
-	i = snprintf(p, len, " srcipxsock %s 0x%04x",
-		     fr_int2str(filterCompare, filter->u.ipx.srcSocComp, "??"),
-		     ntohs(filter->u.ipx.src.socket));
 	p += i;
-	len -= i;
-      }
-    }
+	outlen -= i;
 
-    /* same for destination */
-    if (filter->u.ipx.dst.net) {
-      i = snprintf(p, len, " dstipxnet 0x%04x dstipxnode 0x%02x%02x%02x%02x%02x%02x",
-		  (unsigned int)ntohl(filter->u.ipx.dst.net),
-		  filter->u.ipx.dst.node[0], filter->u.ipx.dst.node[1],
-		  filter->u.ipx.dst.node[2], filter->u.ipx.dst.node[3],
-		  filter->u.ipx.dst.node[4], filter->u.ipx.dst.node[5]);
-      p += i;
-      len -= i;
+	/*
+	*	Handle IP filters
+	*/
+	if (filter->type == RAD_FILTER_IP) {
 
-      if (filter->u.ipx.dstSocComp > RAD_NO_COMPARE) {
-	i = snprintf(p, len, " dstipxsock %s 0x%04x",
-		     fr_int2str(filterCompare, filter->u.ipx.dstSocComp, "??"),
-		     ntohs(filter->u.ipx.dst.socket));
-	p += i;
-	len -= i;
-      }
-    }
+		if (filter->u.ip.srcip) {
+			i = snprintf(p, outlen, " srcip %d.%d.%d.%d/%d",
+				     ((uint8_t const *) &filter->u.ip.srcip)[0],
+				     ((uint8_t const *) &filter->u.ip.srcip)[1],
+				     ((uint8_t const *) &filter->u.ip.srcip)[2],
+				     ((uint8_t const *) &filter->u.ip.srcip)[3],
+				     filter->u.ip.srcmask);
+			p += i;
+			outlen -= i;
+		}
 
+		if (filter->u.ip.dstip) {
+			i = snprintf(p, outlen, " dstip %d.%d.%d.%d/%d",
+				     ((uint8_t const *) &filter->u.ip.dstip)[0],
+				     ((uint8_t const *) &filter->u.ip.dstip)[1],
+				     ((uint8_t const *) &filter->u.ip.dstip)[2],
+				     ((uint8_t const *) &filter->u.ip.dstip)[3],
+				     filter->u.ip.dstmask);
+			p += i;
+			outlen -= i;
+		}
 
-  } else if (filter->type == RAD_FILTER_GENERIC) {
-    int count;
+		i = snprintf(p, outlen, " %s", fr_int2str(filterProtoName, filter->u.ip.proto, "??"));
+		p += i;
+		outlen -= i;
 
-    i = snprintf(p, len, " %u ", (unsigned int) ntohs(filter->u.generic.offset));
-    p += i;
-    i -= len;
+		if (filter->u.ip.srcPortComp > RAD_NO_COMPARE) {
+			i = snprintf(p, outlen, " srcport %s %d",
+				     fr_int2str(filterCompare, filter->u.ip.srcPortComp, "??"),
+				     ntohs(filter->u.ip.srcport));
+			p += i;
+			outlen -= i;
+		}
 
-    /* show the mask */
-    for (count = 0; count < ntohs(filter->u.generic.len); count++) {
-      i = snprintf(p, len, "%02x", filter->u.generic.mask[count]);
-      p += i;
-      len -= i;
-    }
+		if (filter->u.ip.dstPortComp > RAD_NO_COMPARE) {
+			i = snprintf(p, outlen, " dstport %s %d",
+				     fr_int2str(filterCompare, filter->u.ip.dstPortComp, "??"),
+				     ntohs(filter->u.ip.dstport));
+			p += i;
+			outlen -= i;
+		}
 
-    strcpy(p, " ");
-    p++;
-    len--;
+		if (filter->u.ip.established) {
+			i = snprintf(p, outlen, " est");
+			p += i;
+		}
 
-    /* show the value */
-    for (count = 0; count < ntohs(filter->u.generic.len); count++) {
-      i = snprintf(p, len, "%02x", filter->u.generic.value[count]);
-      p += i;
-      len -= i;
-    }
+		/*
+		 *	Handle IPX filters
+		 */
+	} else if (filter->type == RAD_FILTER_IPX) {
+		/* print for source */
+		if (filter->u.ipx.src.net) {
+			i = snprintf(p, outlen, " srcipxnet 0x%04x srcipxnode 0x%02x%02x%02x%02x%02x%02x",
+				  (unsigned int)ntohl(filter->u.ipx.src.net),
+				  filter->u.ipx.src.node[0], filter->u.ipx.src.node[1],
+				  filter->u.ipx.src.node[2], filter->u.ipx.src.node[3],
+				  filter->u.ipx.src.node[4], filter->u.ipx.src.node[5]);
+			p += i;
+			outlen -= i;
 
-    i = snprintf(p, len, " %s", (filter->u.generic.compNeq) ? "!=" : "==");
-    p += i;
-    len -= i;
+			if (filter->u.ipx.srcSocComp > RAD_NO_COMPARE) {
+				i = snprintf(p, outlen, " srcipxsock %s 0x%04x",
+					     fr_int2str(filterCompare, filter->u.ipx.srcSocComp, "??"),
+					     ntohs(filter->u.ipx.src.socket));
+				p += i;
+				outlen -= i;
+			}
+		}
 
-    if (filter->u.generic.more != 0) {
-      i = snprintf(p, len, " more");
-      p += i;
-      len -= i;
-    }
-  }
+		/* same for destination */
+		if (filter->u.ipx.dst.net) {
+			i = snprintf(p, outlen, " dstipxnet 0x%04x dstipxnode 0x%02x%02x%02x%02x%02x%02x",
+				  (unsigned int)ntohl(filter->u.ipx.dst.net),
+				  filter->u.ipx.dst.node[0], filter->u.ipx.dst.node[1],
+				  filter->u.ipx.dst.node[2], filter->u.ipx.dst.node[3],
+				  filter->u.ipx.dst.node[4], filter->u.ipx.dst.node[5]);
+			p += i;
+			outlen -= i;
 
-  *(p++) = '"';
-  *p = '\0';
+			if (filter->u.ipx.dstSocComp > RAD_NO_COMPARE) {
+				i = snprintf(p, outlen, " dstipxsock %s 0x%04x",
+					     fr_int2str(filterCompare, filter->u.ipx.dstSocComp, "??"),
+					     ntohs(filter->u.ipx.dst.socket));
+				p += i;
+			}
+		}
+	} else if (filter->type == RAD_FILTER_GENERIC) {
+		int count;
+
+		i = snprintf(p, outlen, " %u ", (unsigned int) ntohs(filter->u.generic.offset));
+		p += i;
+
+		/* show the mask */
+		for (count = 0; count < ntohs(filter->u.generic.len); count++) {
+			i = snprintf(p, outlen, "%02x", filter->u.generic.mask[count]);
+			p += i;
+			outlen -= i;
+		}
+
+		strcpy(p, " ");
+		p++;
+		outlen--;
+
+		/* show the value */
+		for (count = 0; count < ntohs(filter->u.generic.len); count++) {
+			i = snprintf(p, outlen, "%02x", filter->u.generic.value[count]);
+			p += i;
+			outlen -= i;
+		}
+
+		i = snprintf(p, outlen, " %s", (filter->u.generic.compNeq) ? "!=" : "==");
+		p += i;
+		outlen -= i;
+
+		if (filter->u.generic.more != 0) {
+			i = snprintf(p, outlen, " more");
+			p += i;
+		}
+	}
+
+	if (quote > 0) {
+		*(p++) = (char) quote;
+	}
+	*p = '\0';
 }
+
+#endif
