@@ -1,7 +1,7 @@
 /*
  * acct.c	Accounting routines.
  *
- * Version:	$Id: acct.c,v 1.46 2008/04/18 09:29:49 aland Exp $
+ * Version:	$Id: 90a0dd838bb8beaeae758dd0220f8bc4de677e74 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,12 +23,12 @@
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
 
-#include <freeradius-devel/ident.h>
-RCSID("$Id: acct.c,v 1.46 2008/04/18 09:29:49 aland Exp $")
+RCSID("$Id: 90a0dd838bb8beaeae758dd0220f8bc4de677e74 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
 
+#ifdef WITH_ACCOUNTING
 /*
  *	rad_accounting: call modules.
  *
@@ -39,82 +39,90 @@ int rad_accounting(REQUEST *request)
 {
 	int result = RLM_MODULE_OK;
 
+
+#ifdef WITH_PROXY
+#define WAS_PROXIED (request->proxy)
+#else
+#define WAS_PROXIED (0)
+#endif
+
 	/*
 	 *	Run the modules only once, before proxying.
 	 */
-	if (!request->proxy) {
+	if (!WAS_PROXIED) {
 		VALUE_PAIR	*vp;
 		int		acct_type = 0;
 
 		result = module_preacct(request);
 		switch (result) {
-			/*
-			 *	The module has a number of OK return codes.
-			 */
-			case RLM_MODULE_NOOP:
-			case RLM_MODULE_OK:
-			case RLM_MODULE_UPDATED:
-				break;
-			/*
-			 *	The module handled the request, stop here.
-			 */
-			case RLM_MODULE_HANDLED:
-				return result;
-			/*
-			 *	The module failed, or said the request is
-			 *	invalid, therefore we stop here.
-			 */
-			case RLM_MODULE_FAIL:
-			case RLM_MODULE_INVALID:
-			case RLM_MODULE_NOTFOUND:
-			case RLM_MODULE_REJECT:
-			case RLM_MODULE_USERLOCK:
-			default:
-				return result;
+		/*
+		 *	The module has a number of OK return codes.
+		 */
+		case RLM_MODULE_NOOP:
+		case RLM_MODULE_OK:
+		case RLM_MODULE_UPDATED:
+			break;
+		/*
+		 *	The module handled the request, stop here.
+		 */
+		case RLM_MODULE_HANDLED:
+			return result;
+		/*
+		 *	The module failed, or said the request is
+		 *	invalid, therefore we stop here.
+		 */
+		case RLM_MODULE_FAIL:
+		case RLM_MODULE_INVALID:
+		case RLM_MODULE_NOTFOUND:
+		case RLM_MODULE_REJECT:
+		case RLM_MODULE_USERLOCK:
+		default:
+			return result;
 		}
 
 		/*
 		 *	Do the data storage before proxying. This is to ensure
 		 *	that we log the packet, even if the proxy never does.
 		 */
-		vp = pairfind(request->config_items, PW_ACCT_TYPE);
+		vp = fr_pair_find_by_num(request->config, PW_ACCT_TYPE, 0, TAG_ANY);
 		if (vp) {
-			DEBUG2("  Found Acct-Type %s", vp->vp_strvalue);
 			acct_type = vp->vp_integer;
+			DEBUG2("  Found Acct-Type %s",
+			       dict_valnamebyattr(PW_ACCT_TYPE, 0, acct_type));
 		}
-		result = module_accounting(acct_type, request);
+		result = process_accounting(acct_type, request);
 		switch (result) {
-			/*
-			 *	In case the accounting module returns FAIL,
-			 *	it's still useful to send the data to the
-			 *	proxy.
-			 */
-			case RLM_MODULE_FAIL:
-			case RLM_MODULE_NOOP:
-			case RLM_MODULE_OK:
-			case RLM_MODULE_UPDATED:
-				break;
-			/*
-			 *	The module handled the request, don't reply.
-			 */
-			case RLM_MODULE_HANDLED:
-				return result;
-			/*
-			 *	Neither proxy, nor reply to invalid requests.
-			 */
-			case RLM_MODULE_INVALID:
-			case RLM_MODULE_NOTFOUND:
-			case RLM_MODULE_REJECT:
-			case RLM_MODULE_USERLOCK:
-			default:
-				return result;
+		/*
+		 *	In case the accounting module returns FAIL,
+		 *	it's still useful to send the data to the
+		 *	proxy.
+		 */
+		case RLM_MODULE_FAIL:
+		case RLM_MODULE_NOOP:
+		case RLM_MODULE_OK:
+		case RLM_MODULE_UPDATED:
+			break;
+		/*
+		 *	The module handled the request, don't reply.
+		 */
+		case RLM_MODULE_HANDLED:
+			return result;
+		/*
+		 *	Neither proxy, nor reply to invalid requests.
+		 */
+		case RLM_MODULE_INVALID:
+		case RLM_MODULE_NOTFOUND:
+		case RLM_MODULE_REJECT:
+		case RLM_MODULE_USERLOCK:
+		default:
+			return result;
 		}
 
 		/*
 		 *	Maybe one of the preacct modules has decided
 		 *	that a proxy should be used.
 		 */
-		if ((vp = pairfind(request->config_items, PW_PROXY_TO_REALM))) {
+		if ((vp = fr_pair_find_by_num(request->config, PW_PROXY_TO_REALM, 0, TAG_ANY))) {
 			REALM *realm;
 
 			/*
@@ -124,7 +132,7 @@ int rad_accounting(REQUEST *request)
 			realm = realm_find2(vp->vp_strvalue);
 			if (realm && !realm->acct_pool) {
 				DEBUG("rad_accounting: Cancelling proxy to realm %s, as it is a LOCAL realm.", realm->name);
-				pairdelete(&request->config_items, PW_PROXY_TO_REALM);
+				fr_pair_delete_by_num(&request->config, PW_PROXY_TO_REALM, 0, TAG_ANY);
 			} else {
 				/*
 				 *	Don't reply to the NAS now because
@@ -136,6 +144,13 @@ int rad_accounting(REQUEST *request)
 		}
 	}
 
+#ifdef WITH_PROXY
+	/*
+	 *	We didn't see a reply to the proxied request.  Fail.
+	 */
+	if (request->proxy && !request->proxy_reply) return RLM_MODULE_FAIL;
+#endif
+
 	/*
 	 *	We get here IF we're not proxying, OR if we've
 	 *	received the accounting reply from the end server,
@@ -145,30 +160,27 @@ int rad_accounting(REQUEST *request)
 	 *      Accounting-Response.
 	 */
 	switch (result) {
-		/*
-		 *	Send back an ACK to the NAS.
-		 */
-		case RLM_MODULE_OK:
-		case RLM_MODULE_UPDATED:
-			request->reply->code = PW_ACCOUNTING_RESPONSE;
-			break;
-		/*
-		 *	The module handled the request, don't reply.
-		 */
-		case RLM_MODULE_HANDLED:
-			break;
-		/*
-		 *	Failed to log or to proxy the accounting data,
-		 *	therefore don't reply to the NAS.
-		 */
-		case RLM_MODULE_FAIL:
-		case RLM_MODULE_INVALID:
-		case RLM_MODULE_NOOP:
-		case RLM_MODULE_NOTFOUND:
-		case RLM_MODULE_REJECT:
-		case RLM_MODULE_USERLOCK:
-		default:
-			break;
+	/*
+	 *	Send back an ACK to the NAS.
+	 */
+	case RLM_MODULE_OK:
+	case RLM_MODULE_UPDATED:
+		request->reply->code = PW_CODE_ACCOUNTING_RESPONSE;
+		break;
+
+	/*
+	 *	Failed to log or to proxy the accounting data,
+	 *	therefore don't reply to the NAS.
+	 */
+	case RLM_MODULE_FAIL:
+	case RLM_MODULE_INVALID:
+	case RLM_MODULE_NOOP:
+	case RLM_MODULE_NOTFOUND:
+	case RLM_MODULE_REJECT:
+	case RLM_MODULE_USERLOCK:
+	default:
+		break;
 	}
 	return result;
 }
+#endif
