@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: 08b00348b289ee982b188b8e657a3dd5a6d56b88 $
+ * Version:	$Id: 6d6441a55b743b18c38effb19abaefccda577e03 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-RCSID("$Id: 08b00348b289ee982b188b8e657a3dd5a6d56b88 $")
+RCSID("$Id: 6d6441a55b743b18c38effb19abaefccda577e03 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/parser.h>
@@ -461,8 +461,7 @@ static int file_callback(void *ctx, void *data)
 	/*
 	 *	The file changed, we'll need to re-read it.
 	 */
-	if (buf.st_mtime != file->buf.st_mtime) {
-
+	if (file->buf.st_mtime != buf.st_mtime) {
 		if (cb->callback(cb->modules, file->cs)) {
 			cb->rcode |= CF_FILE_MODULE;
 			DEBUG3("HUP: Changed module file %s", file->filename);
@@ -470,6 +469,12 @@ static int file_callback(void *ctx, void *data)
 			DEBUG3("HUP: Changed config file %s", file->filename);
 			cb->rcode |= CF_FILE_CONFIG;
 		}
+
+		/*
+		 *	Presume that the file will be immediately
+		 *	re-read, so we update the mtime appropriately.
+		 */
+		file->buf.st_mtime = buf.st_mtime;
 	}
 
 	return 0;
@@ -1088,7 +1093,7 @@ static char const *cf_expand_variables(char const *cf, int *lineno,
 			end = strchr(ptr, '}');
 			if (end == NULL) {
 				*p = '\0';
-				INFO("%s[%d]: Variable expansion missing }",
+				ERROR("%s[%d]: Variable expansion missing }",
 				       cf, *lineno);
 				return NULL;
 			}
@@ -1233,7 +1238,7 @@ static char const *cf_expand_variables(char const *cf, int *lineno,
 			end = strchr(ptr, '}');
 			if (end == NULL) {
 				*p = '\0';
-				INFO("%s[%d]: Environment variable expansion missing }",
+				ERROR("%s[%d]: Environment variable expansion missing }",
 				       cf, *lineno);
 				return NULL;
 			}
@@ -1316,29 +1321,32 @@ static inline int fr_item_validate_ipaddr(CONF_SECTION *cs, char const *name, PW
 	case PW_TYPE_COMBO_IP_ADDR:
 		switch (ipaddr->af) {
 		case AF_INET:
-		if (ipaddr->prefix != 32) {
-			ERROR("Invalid IPv4 mask length \"/%i\".  Only \"/32\" permitted for non-prefix types",
-			      ipaddr->prefix);
+			if (ipaddr->prefix == 32) return 0;
 
-			return -1;
-		}
+			cf_log_err(&(cs->item), "Invalid IPv4 mask length \"/%i\".  Only \"/32\" permitted for non-prefix types",
+				   ipaddr->prefix);
 			break;
 
 		case AF_INET6:
-		if (ipaddr->prefix != 128) {
-			ERROR("Invalid IPv6 mask length \"/%i\".  Only \"/128\" permitted for non-prefix types",
-			      ipaddr->prefix);
+			if (ipaddr->prefix == 128) return 0;
 
-			return -1;
-		}
+			cf_log_err(&(cs->item), "Invalid IPv6 mask length \"/%i\".  Only \"/128\" permitted for non-prefix types",
+				   ipaddr->prefix);
 			break;
 
+
 		default:
-			return -1;
+			cf_log_err(&(cs->item), "Unknown address (%d) family passed for parsing IP address.", ipaddr->af);
+			break;
 		}
+
+		return -1;
+
 	default:
-		return 0;
+		break;
 	}
+
+	return 0;
 }
 
 /** Parses a #CONF_PAIR into a C data type, with a default value.
@@ -1416,7 +1424,10 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 	char buffer[8192];
 	CONF_ITEM *c_item;
 
-	if (!cs) return -1;
+	if (!cs) {
+		cf_log_err(&(cs->item), "No enclosing section for configuration item \"%s\"", name);
+		return -1;
+	}
 
 	c_item = &cs->item;
 
@@ -1675,10 +1686,12 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 		 *	server startup.
 		 */
 		if (*q && file_input && !cf_file_check(cs, *q, true)) {
+			cf_log_err(&(cs->item), "Failed parsing configuration item \"%s\"", name);
 			return -1;
 		}
 
 		if (*q && file_exists && !cf_file_check(cs, *q, false)) {
+			cf_log_err(&(cs->item), "Failed parsing configuration item \"%s\"", name);
 			return -1;
 		}
 		break;
@@ -1688,7 +1701,8 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 		ipaddr = data;
 
 		if (fr_pton4(ipaddr, value, -1, true, false) < 0) {
-			ERROR("%s", fr_strerror());
+		failed:
+			cf_log_err(&(cs->item), "Failed parsing configuration item \"%s\" - %s", name, fr_strerror());
 			return -1;
 		}
 		if (fr_item_validate_ipaddr(cs, name, type, value, ipaddr) < 0) return -1;
@@ -1698,10 +1712,7 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 	case PW_TYPE_IPV6_PREFIX:
 		ipaddr = data;
 
-		if (fr_pton6(ipaddr, value, -1, true, false) < 0) {
-			ERROR("%s", fr_strerror());
-			return -1;
-		}
+		if (fr_pton6(ipaddr, value, -1, true, false) < 0) goto failed;
 		if (fr_item_validate_ipaddr(cs, name, type, value, ipaddr) < 0) return -1;
 		break;
 
@@ -1709,10 +1720,7 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 	case PW_TYPE_COMBO_IP_PREFIX:
 		ipaddr = data;
 
-		if (fr_pton(ipaddr, value, -1, AF_UNSPEC, true) < 0) {
-			ERROR("%s", fr_strerror());
-			return -1;
-		}
+		if (fr_pton(ipaddr, value, -1, AF_UNSPEC, true) < 0) goto failed;
 		if (fr_item_validate_ipaddr(cs, name, type, value, ipaddr) < 0) return -1;
 		break;
 
@@ -1730,7 +1738,7 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 			len = strlen(end + 1);
 
 			if (len > 6) {
-				ERROR("Too much precision for timeval");
+				cf_log_err(&(cs->item), "Too much precision for timeval");
 				return -1;
 			}
 
@@ -1761,7 +1769,7 @@ int cf_item_parse(CONF_SECTION *cs, char const *name, unsigned int type, void *d
 		rad_assert(type > PW_TYPE_INVALID);
 		rad_assert(type < PW_TYPE_MAX);
 
-		ERROR("type '%s' is not supported in the configuration files",
+		cf_log_err(&(cs->item), "type '%s' is not supported in the configuration files",
 		       fr_int2str(dict_attr_types, type, "?Unknown?"));
 		return -1;
 	} /* switch over variable type */
