@@ -15,7 +15,7 @@
  */
 
 /**
- * $Id: 96dd1cdf9d17e13022e2dfc6759b6d98a501f3ef $
+ * $Id: f4287a97330622fb5f905f03767a3e0412c4bb62 $
  * @file rlm_mschap.c
  * @brief Implemented mschap authentication.
  *
@@ -23,7 +23,7 @@
  */
 
 /*  MPPE support from Takahiro Wagatsuma <waga@sic.shibaura-it.ac.jp> */
-RCSID("$Id: 96dd1cdf9d17e13022e2dfc6759b6d98a501f3ef $")
+RCSID("$Id: f4287a97330622fb5f905f03767a3e0412c4bb62 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -1555,7 +1555,7 @@ static rlm_rcode_t CC_HINT(nonnull) mod_authenticate(void *instance, REQUEST *re
 	VALUE_PAIR *response = NULL;
 	VALUE_PAIR *cpw = NULL;
 	VALUE_PAIR *password = NULL;
-	VALUE_PAIR *lm_password, *nt_password, *smb_ctrl;
+	VALUE_PAIR *nt_password, *smb_ctrl;
 	VALUE_PAIR *username;
 	uint8_t nthashhash[NT_DIGEST_LENGTH];
 	char msch2resp[42];
@@ -1666,57 +1666,6 @@ static rlm_rcode_t CC_HINT(nonnull) mod_authenticate(void *instance, REQUEST *re
 		}
 	}
 
-	/*
-	 *	Or an LM-Password.
-	 */
-	lm_password = fr_pair_find_by_num(request->config, PW_LM_PASSWORD, 0, TAG_ANY);
-	if (lm_password) {
-		VERIFY_VP(lm_password);
-
-		switch (lm_password->vp_length) {
-		case LM_DIGEST_LENGTH:
-			RDEBUG2("Found LM-Password");
-			break;
-
-		/* 0x */
-		case 34:
-		case 32:
-			RWDEBUG("LM-Password has not been normalized by the 'pap' module (likely still in hex format).  "
-				"Authentication may fail");
-			lm_password = NULL;
-			break;
-
-		default:
-			RWDEBUG("LM-Password found but incorrect length, expected " STRINGIFY(LM_DIGEST_LENGTH)
-				" bytes got %zu bytes.  Authentication may fail", lm_password->vp_length);
-			lm_password = NULL;
-			break;
-		}
-	}
-	/*
-	 *	... or a Cleartext-Password, which we now transform into an LM-Password
-	 */
-	if (!lm_password) {
-		if (password) {
-			RDEBUG2("Found Cleartext-Password, hashing to create LM-Password");
-			lm_password = pair_make_config("LM-Password", NULL, T_OP_EQ);
-			if (!lm_password) {
-				RERROR("No memory");
-			} else {
-				uint8_t *p;
-
-				lm_password->vp_length = LM_DIGEST_LENGTH;
-				lm_password->vp_octets = p = talloc_array(lm_password, uint8_t, lm_password->vp_length);
-				smbdes_lmpwdhash(password->vp_strvalue, p);
-			}
-		/*
-		 *	Only complain if we don't have NT-Password
-		 */
-		} else if ((auth_method == AUTH_INTERNAL) && !nt_password) {
-			RWDEBUG2("No Cleartext-Password configured.  Cannot create LM-Password");
-		}
-	}
-
 	cpw = fr_pair_find_by_num(request->packet->vps, PW_MSCHAP2_CPW, VENDORPEC_MICROSOFT, TAG_ANY);
 	if (cpw) {
 		/*
@@ -1903,9 +1852,8 @@ static rlm_rcode_t CC_HINT(nonnull) mod_authenticate(void *instance, REQUEST *re
 			password = nt_password;
 			offset = 26;
 		} else {
-			RDEBUG2("Client is using MS-CHAPv1 with LM-Password");
-			password = lm_password;
-			offset = 2;
+			REDEBUG2("Client is using MS-CHAPv1 with unsupported method LM-Password");
+			return RLM_MODULE_FAIL;
 		}
 
 		/*
@@ -1994,8 +1942,8 @@ static rlm_rcode_t CC_HINT(nonnull) mod_authenticate(void *instance, REQUEST *re
 		 *  No "known good" NT-Password attribute.  Try to do
 		 *  OpenDirectory authentication.
 		 *
-		 *  If OD determines the user is an AD user it will return noop, which
-		 *  indicates the auth process should continue directly to AD.
+		 *  If OD determines the user is an OD user it will return noop, which
+		 *  indicates the auth process should continue directly to OD.
 		 *  Otherwise OD will determine auth success/fail.
 		 */
 		if (!nt_password && inst->open_directory) {
@@ -2066,9 +2014,6 @@ static rlm_rcode_t CC_HINT(nonnull) mod_authenticate(void *instance, REQUEST *re
 		if (mschap_version == 1) {
 			RDEBUG2("adding MS-CHAPv1 MPPE keys");
 			memset(mppe_sendkey, 0, 32);
-			if (lm_password) {
-				memcpy(mppe_sendkey, lm_password->vp_octets, 8);
-			}
 
 			/*
 			 *	According to RFC 2548 we
